@@ -14,6 +14,7 @@ from __future__ import annotations
 import hashlib
 import json
 import pathlib
+import shlex
 from typing import Dict, List, NamedTuple, Optional, Union
 from . import language
 from . import message
@@ -34,12 +35,12 @@ class CodeLineOrigin(NamedTuple):
 
 
 
-class Code(object):
+class CodeCollection(object):
     '''
     Base class for sessions and sources.  Never instantiated.
     '''
     def __init__(self, code_key: CodeKey, *, code_defaults: Optional[Dict[str, Union[bool, str]]]):
-        if type(self) is Code:
+        if type(self) is CodeCollection:
             raise NotImplementedError
         self.key: CodeKey = code_key
         self.lang: Optional[str] = code_key.lang
@@ -68,7 +69,7 @@ class Code(object):
 
 
 
-class Source(Code):
+class Source(CodeCollection):
     '''
     An ordered collection of code chunks that is typically displayed but never
     executed.  May be exported as a single file of source code.
@@ -92,7 +93,7 @@ class Source(Code):
         Append a code chunk to internal code chunk list.  Check code chunk
         options for validity and update chunk summary data.
         '''
-        code_chunk.source = self
+        code_chunk.code_collection = self
         code_chunk.index = len(self.code_chunks)
         code_chunk.errors.register_status(self.status)
         code_chunk.warnings.register_status(self.status)
@@ -123,7 +124,7 @@ class Source(Code):
 
 
 
-class Session(Code):
+class Session(CodeCollection):
     '''
     Code chunks comprising a session.
     '''
@@ -133,7 +134,9 @@ class Session(Code):
         super().__init__(code_key, code_defaults=code_defaults)
 
         self.lang_def: Optional[language.Language] = None
-        self.executable: Optional[pathlib.Path] = None
+        self.executable: Optional[str] = None
+        self.executable_opts: Optional[list[str]] = None
+        self.args: Optional[list[str]] = None
         self.live_output: Optional[bool] = None
         self.repl: Optional[bool] = None
         self.jupyter_kernel: Optional[str] = None
@@ -185,7 +188,7 @@ class Session(Code):
         Append a code chunk to internal code chunk list.  Check code chunk
         options for validity and update chunk summary data.
         '''
-        code_chunk.session = self
+        code_chunk.code_collection = self
         code_chunk.index = len(self.code_chunks)
         code_chunk.errors.register_status(self.status)
         code_chunk.warnings.register_status(self.status)
@@ -202,7 +205,29 @@ class Session(Code):
                     msg = f'Language definition for "{self.lang}" does not exist'
                     self.errors.append(message.SysConfigError(msg))
                 else:
-                    self.executable = pathlib.Path(first_chunk_options.get('executable', self.lang_def.executable)).expanduser()
+                    try:
+                        raw_executable = first_chunk_options['executable']
+                    except KeyError:
+                        executable = self.lang_def.executable
+                    else:
+                        executable = pathlib.Path(raw_executable).expanduser().as_posix()
+                        if raw_executable.startswith('./') or raw_executable.startswith('.\\'):
+                            executable = f'./{executable}'
+                    self.executable = executable
+                    try:
+                        raw_executable_opts = first_chunk_options['executable_opts']
+                    except KeyError:
+                        executable_opts = self.lang_def.executable_opts
+                    else:
+                        executable_opts = shlex.split(raw_executable_opts)
+                    self.executable_opts = executable_opts
+                    try:
+                        raw_args = first_chunk_options['args']
+                    except KeyError:
+                        args = self.lang_def.args
+                    else:
+                        args = shlex.split(raw_args)
+                    self.args = args
                     self.repl = self.lang_def.repl
             live_output = first_chunk_options.get('live_output')
             if live_output is not None:
@@ -271,7 +296,7 @@ class Session(Code):
             else:
                 msg = 'Final code chunk cannot have "complete" value "false"'
                 last_cc.errors.append(message.SourceError(msg))
-        if self.status.has_errors:
+        if self.status.prevent_exec:
             # Hashes and line numbers are only needed if code will indeed be
             # executed.  It is impossible to determine these in the case of
             # errors like copy errors which leave code for one or more chunks
@@ -292,7 +317,9 @@ class Session(Code):
         if self.executable:
             hashed_options = {
                 'session': self.name,
-                'executable': self.executable.as_posix(),
+                'executable': self.executable,
+                'executable_opts': self.executable_opts,
+                'args': self.args,
             }
         elif self.jupyter_kernel:
             hashed_options = {
@@ -324,10 +351,10 @@ class Session(Code):
             }
             hasher.update(json.dumps(cc_options).encode('utf8'))
             hasher.update(hasher.digest())
-            code_bytes = cc.code.encode('utf8')
+            code_bytes = cc.code_str.encode('utf8')
             hasher.update(code_bytes)
             hasher.update(hasher.digest())
-            code_len += len(cc.code) + 1  # +1 for omitted trailing newline
+            code_len += len(cc.code_str) + 1  # +1 for omitted trailing newline
         self.hash = f'{hasher.hexdigest()}_{code_len}'
         self.hash_root = self.temp_suffix = hasher.hexdigest()[:16]
         self.run_delim_hash = hasher.hexdigest()[:64]
@@ -410,7 +437,7 @@ class Session(Code):
                         expr_start_delim=expr_start_delim,
                         expr_end_delim=expr_end_delim,
                         temp_suffix=self.temp_suffix,
-                        code=cc.code,
+                        code=cc.code_str,
                     )
                     if not self.lang_def.chunk_wrapper_code_indent:
                         run_code_list.append(expr_code)
@@ -423,7 +450,7 @@ class Session(Code):
                     self.run_code_to_origins[line_number] = CodeLineOrigin(chunk=cc, line_number=1)
                     run_code_line_number += self.lang_def.inline_expression_formatter_n_lines
                 else:
-                    run_code_list.append(f'{self.lang_def.chunk_wrapper_code_indent}{cc.code}\n')
+                    run_code_list.append(f'{self.lang_def.chunk_wrapper_code_indent}{cc.code_str}\n')
                     self.run_code_to_origins[run_code_line_number] = CodeLineOrigin(chunk=cc, line_number=1)
                     run_code_line_number += 1
             else:
