@@ -67,8 +67,9 @@ class Converter(object):
                  from_format: Optional[str]=None,
                  code_defaults: Optional[Dict[str, Union[bool, str]]]=None,
                  session_defaults: Optional[Dict[str, Union[bool, str]]]=None,
+                 no_execute: bool=False,
+                 only_code_output: Optional[str]=None,
                  synctex: bool=False):
-        self._progress = Progress()
 
         if not all(isinstance(x, bool) for x in (cross_origin_sessions, expanduser, expandvars)):
             raise TypeError
@@ -184,6 +185,18 @@ class Converter(object):
         self.origin_paths_for_cache = origin_paths_for_cache
         self.cache_key = cache_key_hasher.hexdigest()[:16]
 
+        if not isinstance(no_execute, bool):
+            raise TypeError
+        self.no_execute = no_execute
+
+        if only_code_output is None:
+            pass
+        elif not isinstance(only_code_output, str):
+            raise TypeError
+        elif only_code_output not in ('codebraid_preview',):
+            raise ValueError
+        self.only_code_output = only_code_output
+
         self._io_map = False
         if not isinstance(synctex, bool):
             raise TypeError
@@ -191,6 +204,8 @@ class Converter(object):
         if synctex:
             self._io_map = True
 
+        self._progress = Progress(self.only_code_output)
+
         self.code_chunks = []
         self.code_processor: Optional[codeprocessors.CodeProcessor] = None
 
@@ -220,8 +235,10 @@ class Converter(object):
     def code_braid(self):
         self.extract_code_chunks()
         self.process_code_chunks()
-        self.exec_code_chunks()
-        self.postprocess_code_chunks()
+        if self.no_execute:
+            pass
+        else:
+            self.exec_code_chunks()
 
     def extract_code_chunks(self):
         self._progress.parse_start()
@@ -246,7 +263,8 @@ class Converter(object):
             origin_paths_for_cache=self.origin_paths_for_cache,
             code_defaults=self.code_defaults,
             session_defaults=self.session_defaults,
-            progress=self._progress
+            progress=self._progress,
+            only_code_output=self.only_code_output,
         )
         self.code_processor.process()
 
@@ -255,14 +273,6 @@ class Converter(object):
         self.code_processor.exec()
         self._progress.exec_end()
 
-    def postprocess_code_chunks(self):
-        self._progress.postprocess_start()
-        self._postprocess_code_chunks()
-        self._progress.postprocess_end()
-
-    def _postprocess_code_chunks(self):
-        raise NotImplementedError
-
     def convert(self, *, to_format, **kwargs):
         self._progress.convert_start()
         self._convert(to_format=to_format, **kwargs)
